#!/usr/bin/env python
# coding: utf-8

"""
    create at 2017/11/19 by allen
"""


from relation import user_role, role_resource
from role import Role
from user import User
from resource import Resource
