#!/usr/bin/env python
# coding: utf-8

"""
    create at 2017/11/6 by allen
"""

from email import EmailType as Email
from ip_address import IpAddressType as IpAddress
from password import PasswordType as Password

__all__ = (
    'Email', 'IpAddress', 'Password'
)
